@@ -16,6 +16,11 @@
 */
 #include <config.h>
 
+#ifdef _WIN32
+/* Avoid conflicting types for INT32 */
+#define QGLOBAL_H
+#endif
+
 #include <stdio.h>
 #include <jpeglib.h>
 
@@ -108,7 +113,7 @@ void jpeg_encoder_destroy(JpegEncoderContext* encoder)
 
 static void convert_RGB16_to_RGB24(void *line, int width, uint8_t **out_line)
 {
-    uint16_t *src_line = line;
+    uint16_t *src_line = (uint16_t *) line;
     uint8_t *out_pix;
     int x;
 
@@ -129,7 +134,7 @@ static void convert_BGR24_to_RGB24(void *in_line, int width, uint8_t **out_line)
 {
     int x;
     uint8_t *out_pix;
-    uint8_t *line = in_line;
+    uint8_t *line = (uint8_t *) in_line;
     spice_assert(out_line && *out_line);
 
     out_pix = *out_line;
@@ -144,7 +149,7 @@ static void convert_BGR24_to_RGB24(void *in_line, int width, uint8_t **out_line)
 
 static void convert_BGRX32_to_RGB24(void *line, int width, uint8_t **out_line)
 {
-    uint32_t *src_line = line;
+    uint32_t *src_line = (uint32_t *) line;
     uint8_t *out_pix;
     int x;
 
